using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.WindowsMobile.PocketOutlook;

namespace GMail2PocketOutlookLib
{
    /// <summary>
    /// Class that handles the import process
    /// </summary>
    public class Importer
    {
        #region Fields

        private OutlookSession outlookSession = null;

        private GMailCSVReader csvReader = null;

        private GMailContact[] newContacts = null;
        private ContactCollection existingContacts = null;

        #endregion

        #region Properties

        /// <summary>
        /// Returns the number of existing contacts
        /// </summary>
        public int ExistingContactsCount
        {
            get
            {
                if (existingContacts != null)
                {
                    return existingContacts.Count;
                }
                else
                {
                    throw new NullReferenceException("GetData() Method must be called first.");
                }
            }
        }

        /// <summary>
        /// Returns the number of contacts to import
        /// </summary>
        public int NewContactsCount
        {
            get
            {
                if (newContacts != null)
                {
                    return newContacts.Length;
                }
                else
                {
                    throw new NullReferenceException("GetData() Method must be called first.");
                }
            }
        }

        #endregion

        #region Class Constructor

        /// <summary>
        /// Class Constructor
        /// </summary>
        public Importer()
        {
            outlookSession = new OutlookSession();
            csvReader = new GMailCSVReader();
        }

        #endregion
        
        #region Public Methods

        /// <summary>
        /// Gets all existing contacts and gathers all data to import into new contacts
        /// </summary>
        /// <param name="Path">Path of the CSV file to use as source for new contacts</param>
        /// <returns>Result of the operation</returns>
        public bool GetData(string Path)
        {
            bool result;

            try
            {
                existingContacts = outlookSession.Contacts.Items;
            }
            catch
            {
                return false;
            }

            result = csvReader.GetData(Path);

            if (result)
                newContacts = csvReader.GMailContactList;

            return result;
        }

        /// <summary>
        /// Imports al gathered data into Outlook's contacts
        /// </summary>
        /// <returns>Result of the operation</returns>
        public bool ImportData()
        {
            string formattedBody;
            Contact newContact;

            if (existingContacts == null || newContacts == null)
            {
                return false;
            }

            for (int index = 0; index < newContacts.Length; index++)
            {
                formattedBody = string.Empty;
                newContact = new Contact();

                newContact.FirstName = newContacts[index].Name;
                newContact.FileAs = newContacts[index].Name;
                newContact.Email1Address = newContacts[index].Email;
                if (!string.IsNullOrEmpty(newContacts[index].Pager_S1))
                {
                    newContact.PagerNumber = newContacts[index].Pager_S1;
                }
                else
                {
                    newContact.PagerNumber = newContacts[index].Pager_S2;
                }
                
                newContact = AddEmailsToContact(newContacts[index], newContact);
                newContact = AddIMsToContact(newContacts[index], newContact);

                if (string.Compare(newContacts[index].Description_S1, "Personal", true) == 0)
                {
                    newContact.HomeTelephoneNumber = newContacts[index].Phone_S1;
                    newContact.BusinessTelephoneNumber = newContacts[index].Phone_S2;
                    newContact.MobileTelephoneNumber = newContacts[index].Mobile_S1;
                    newContact.BusinessTelephoneNumber = newContacts[index].Mobile_S2;
                    newContact.HomeFaxNumber = newContacts[index].Fax_S1;
                    newContact.BusinessFaxNumber = newContacts[index].Fax_S2;
                    newContact.HomeAddressStreet = newContacts[index].Address_S1.Replace("\"", "");
                    newContact.BusinessAddressStreet = newContacts[index].Address_S2.Replace("\"", "");
                    newContact.Body = AddToBodyIfNotEmpty(newContacts[index].Other_S1, newContact.Body);
                    newContact.Body = AddToBodyIfNotEmpty(newContacts[index].Other_S2, newContact.Body);
                }
                else
                {
                    newContact.HomeTelephoneNumber = newContacts[index].Phone_S2;
                    newContact.BusinessTelephoneNumber = newContacts[index].Phone_S1;
                    newContact.MobileTelephoneNumber = newContacts[index].Mobile_S2;
                    newContact.BusinessTelephoneNumber = newContacts[index].Mobile_S1;
                    newContact.HomeFaxNumber = newContacts[index].Fax_S2;
                    newContact.BusinessFaxNumber = newContacts[index].Fax_S1;
                    newContact.HomeAddressStreet = newContacts[index].Address_S2.Replace("\"", "");
                    newContact.BusinessAddressStreet = newContacts[index].Address_S1.Replace("\"", "");
                    newContact.Body = AddToBodyIfNotEmpty(newContacts[index].Other_S2, newContact.Body);
                    newContact.Body = AddToBodyIfNotEmpty(newContacts[index].Other_S1, newContact.Body);
                }
                
                newContact.Body = AddToBodyIfNotEmpty(newContacts[index].Notes, newContact.Body);


                existingContacts.Add(newContact);
            }

            return true;
        }

        /// <summary>
        /// Deletes all existing contacts
        /// </summary>
        /// <remarks>Only deletes PocketPC contacts, not SIM ones</remarks>
        /// <returns>Result of the operation</returns>
        public bool DeleteExistingData()
        {
            try
            {
                existingContacts.Clear();
            }
            catch
            {
                return false;
            }

            return true;
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Splits Instant Messaging addresses
        /// </summary>
        /// <param name="IMData1">IM_S1/IM_S2 field of GMail</param>
        /// <param name="IMData2">IMS1_/IM_S2 field of GMail</param>
        /// <remarks>Adds First the IMData1 IMs, and then the IMData2 ones.
        /// PocketOutlook is limited to 3 IMs, so if more are present, are discarted.</remarks>
        /// <returns>List of IMs to insert in the outlook contact</returns>
        private List<string> splitIMs(string IMData1, string IMData2)
        {
            string data;
            List<string> resultIMs = new List<string>();
            string[] splittedIMs;

            data = IMData1.Replace(" ::: ", "$");
            splittedIMs = data.Split('$');
            foreach (string im in splittedIMs)
            {
                resultIMs.Add(im);
            }
            data = IMData2.Replace(" ::: ", "$");
            splittedIMs = data.Split('$');
            foreach (string im in splittedIMs)
            {
                resultIMs.Add(im);
            }

            if (resultIMs.Count > 3)
            {
                resultIMs.RemoveRange(3, resultIMs.Count - 3);
            }
            else
            {
                while (resultIMs.Count < 3)
                {
                    resultIMs.Add(string.Empty);
                }
            }

            return resultIMs;
        }

        /// <summary>
        /// Adds a field to the outlook contact Body filed, if it's not empty
        /// </summary>
        /// <param name="Field">Field to add</param>
        /// <param name="Body">Body contents</param>
        /// <returns>Body with field added if had data</returns>
        private string AddToBodyIfNotEmpty(string Field, string Body)
        {
            string processedString = Field.Replace("\"", "");

            if (!string.IsNullOrEmpty(processedString))
            {
                Body = Body + processedString + " \r\n";
            }

            return Body;
        }
        
        /// <summary>
        /// Adds emails to a contact, adding first the personal one.
        /// </summary>
        /// <param name="GmailData">GMail data</param>
        /// <param name="ContactData">Outlook data</param>
        /// <returns>Outlook data with the emails added</returns>
        private Contact AddEmailsToContact(GMailContact GmailData, Contact ContactData)
        {
            Contact contact = ContactData;

            // First section is personal info
            if (string.Compare(GmailData.Description_S1, "Personal", true) == 0)
            {
                contact.Email2Address = GmailData.Email_S1;
                contact.Email3Address = GmailData.Email_S2;
            }
            else    // First section is work info, or it is empty
            {
                contact.Email2Address = GmailData.Email_S2;
                contact.Email3Address = GmailData.Email_S1;
            }

            return contact;
        }

        /// <summary>
        /// Adds IMs to a contact.
        /// </summary>
        /// <param name="GmailData">GMail data</param>
        /// <param name="ContactData">Outlook data</param>
        /// <returns>Outlook data with the IMs added</returns>
        private Contact AddIMsToContact(GMailContact GmailData, Contact ContactData)
        {
            Contact contact = ContactData;
            List<string> iMs = new List<string>(3);

            // First section is personal info
            if (string.Compare(GmailData.Description_S1, "Personal", true) == 0)
            {
                iMs = splitIMs(GmailData.IM_S1, GmailData.IM_S2);
            }
            else    // First section is work info, or it is empty
            {
                iMs = splitIMs(GmailData.IM_S2, GmailData.IM_S1);
            }

            contact.IM1Address = iMs[0];
            contact.IM2Address = iMs[1];
            contact.IM3Address = iMs[2];

            return contact;
        }

        #endregion
    }
}
